# Define a way for us to assert something is true
function assert(condition, string)
{
    if (! condition) {
        printf("%s:%d: assertion failed: %s\n",
            FILENAME, FNR, string) > "/dev/stderr"
        _assert_exit = 1
        exit 1
    }
}

# The function we are integrating - sin(x)
function f(x) {
    return sin(x)
}

# Simpson's rule:
# Integrates f(x) from a to b, using n intervals
function simpsons_rule(a,b,n) { 
    # a = lower bound of integral 
    # b = upper bound of integral
    # n = number of partitions on the interval
    assert(n%2==0,"n%2==0")
    s1 = 0.0
    s2 = 0.0
    sum = 0.0
    h = (b - a) / n 
    for(i=1; i<=n; i++) {
        x_i = a * i*h
        if(i%2==0) {
            s1 += f(x_i)
        } else {
            s2 += f(x_i)
        }
    }
    sum = (h/3.0)*(f(a) + f(b) + 2*s1 + 4*s2)
    return sum
}

BEGIN {
    printf("Simpson's Rule Integral Calculation:")
    printf("Integrating sin(x) from 0 to pi/2\n")
    pi = atan2(0, -1)
    a = 0.0
    b = pi/2
    for(n = 10; n<=100000; n*=10) {
        result = simpsons_rule(a,b,n)
        printf("%d segments: int(sin(x)) = %0.8f \n",n,result)
    }
}
