import java.util.LinkedList;
import java.util.Arrays;

public class NQueens { 

    public static final int SIZE = 9;

    ///////////////////////////////////////////////////
    // main 
    //
    public static void main(String[] args) {
        long start = System.nanoTime();
        SolutionSaver s = new SolutionSaver();
        Board b = new Board(SIZE);
        explore(b,0);
        long end = System.nanoTime();
        long duration = end - start;
        System.out.printf("Found %d solutions\n ", SolutionSaver.nSolutions() );
        System.out.printf("Elapsed time %03f s\n ", (duration/1E9) );
    }
    // 
    ///////////////////////////////////////////////////


    /// Placing queens, column-by-column; place the queen in column col. 
    public static void explore(Board b, int col) { 

        if(col >= SIZE ) { 

            // We have reached the end.
            // No conflicts so far means no conflicts period.

            // Save solution in a static class, no instantiation overhead
            SolutionSaver.saveSolution( b.toString() );

		} else {

            int[] attacked = b.getDiagAttacked(col);
            int[] occupied = b.getOccupied(col);

            for(int row=0; row<SIZE; row++) {
                if(occupied[row]!=1 && attacked[row]!=1) { 
                    b.choose(row,col);
                    explore(b,col+1);
                    b.unchoose(row,col);
                }
            }
        }// done with base/recursive cases
    }

}

class Board {
    
    int[] queens;                   // Array to store where queens have been placed.
                                    // The queens array has a length of board_size.
                                    // Each element stores an integer between 1 and N.
                                    // That indicates the row/column.

    int[] occupiedrows;             // Array to mark occupied rows.
                                    // This is how we check for horizontal attacks.

    public static int board_size;

    public Board(int size) {
        this.board_size = size;
        this.queens = new int[size];
        this.occupiedrows = new int[size];
    }

    /** 
     * Get String representation of queen positions.
     * This prints 8 numbers, corresponding to 8 columns.
     * Each number is an integer, 1..(board_size-1), indicating 
     * the row of the queen on that particular column.
     * All queens on first row would be 0 0 0 0 0 0 0 0
     */
    public String toString() { 
        return Arrays.toString(this.queens);
    }

    /// Make the choice of putting a queen on row, col
    public void choose(int row, int col) {
        if( col < this.queens.length && row < this.occupiedrows.length ) {
            this.queens[col] = row;
            this.occupiedrows[row] = 1;
        }
    }

    /// Unmake the choice of putting a queen on row, col
    public void unchoose(int row, int col) {
        if( col < this.queens.length && row < this.occupiedrows.length ) {
            this.queens[col] = 0;
            this.occupiedrows[row] = 0;
        }
    }

    /// Get a list of occupied rows
    public int[] getOccupied(int col) {
        return this.occupiedrows;
    }

    /// Get a list of attacked diagonals
    public int[] getDiagAttacked( int col ) { 

        // 1. Mark invalid squares on diagoonals of already-placed queens
        //
        // 2. For this column, loop over each row where it's legal to place a queen,
        //    and run backtracking on that choice. Then unmake the choice and keep going.

        // Store invalid rows on other queens' diagonals 
        int[] diag = new int[this.board_size];

        // Loop over all of the queens already placed (col-1)
        for(int k = 0; k <= (col-1); k++ ) {

            // We're gonig to place the next queen on col.
            // Find which squares are on diagonals of queen k, 
            // and mark them as impossible.

            // Lower right diag
            int ix1 = this.queens[k] + col - k;
            if(ix1 >= 0 && ix1 < this.board_size ) {
                diag[ix1] = 1;
            }

            // Upper right diag
            int ix2 = this.queens[k] - col + k;
            if(ix2 >= 0 && ix2 < this.board_size ) {
                diag[ix2] = 1;
            }

        }

        return diag;
    }
}


class SolutionSaver {
    private static LinkedList<String> solutions;

    public SolutionSaver() {
        SolutionSaver.solutions = new LinkedList<String>();
    }

    public static void saveSolution(String serialized) { 
        SolutionSaver.solutions.add(serialized);
    }

    public static void printSolutions() { 
        int c = 0;
        for( String sol : solutions ) {
            System.out.printf("Solution %d: %s \n", c, sol);
        }
    }
    public static int nSolutions() { 
        return solutions.size();
    }
}


